; -----------------------------------------------------------------------------
; "Hobbit128" project (BETA version)
; See "README.txt" for credits and detailed information!
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Load all banks just to get their label addresses, although all their contents
; are overwritten immediately afterwards
; -----------------------------------------------------------------------------

include "bank01.asm"
include "bank03.asm"
include "bank04.asm"
include "bank06.asm"
include "bank07.asm"


; -----------------------------------------------------------------------------
; Load entire original game, before "patching" parts of it.
; -----------------------------------------------------------------------------

        ORG     $6000
INCBIN "hobbit_24576.bin"


; -----------------------------------------------------------------------------
; New initialization code
; -----------------------------------------------------------------------------

        ORG     $6c00

        di
        call    $84b9           ; wait for key

        ld      hl, $5800
        ld      de, $5801
        ld      bc, $02ff
        ld      (hl), 0
        ldir                    ; clear screen

        ld      hl, credits
        ld      de, $4000
        call    dzx7            ; decompress credits screen

        call    $84b9           ; wait for key

        ld      de, $5f00
	ld      hl, $b6eb
	ld      bc, $001d
	ldir
	ld      hl, $ca84
	ld      bc, $00bf
	ldir

restart:
        di
	ld      sp, $5eff
        xor     a
        ld      (room05+1), a
        ld      hl, att05night
        ld      (room05+2), hl  ; reset trolls clearing when game restarts

        ld      hl, objs
        ld      de, $c11b
        call    dzx7
        ld      hl, locs
        ld      de, $ba8a
        call    dzx7
        nop
        nop

        ORG     $839d
        jp      restart         ; fix reference to restart address

        ORG     $84b0
        jp      restart         ; fix reference to restart address

        ORG     $90e8
        jp      restart         ; fix reference to restart address


; -----------------------------------------------------------------------------
; Update trolls clearing when day dawns
; -----------------------------------------------------------------------------

        ORG     $a9a8

        ld      a, 4
        ld      (room05+1), a
        ld      hl, att05day
        ld      (room05+2), hl
        ret
REPT 9
        nop
ENDM


; -----------------------------------------------------------------------------
; Area $7f86-$824f originally used for drawing graphics routine, now replaced
; with routine to decompress screens.
; -----------------------------------------------------------------------------

        ORG     $7f86

        pop     af              ; restore location

        push    af
        push    hl
        push    bc
        push    de              ; preserve registers

        ld      ($7f77), a      ; store drawn location

        ld      hl, $5800
        ld      de, $5801
        ld      bc, $01ff
        ld      (hl), $3f
        ldir                    ; clear image area

        ld      de, $4e01
        call    reps            ; consolidate "misty valley" repetitions ($4e-$4f)
        ld      de, $4b02
        call    reps            ; consolidate "path steep" repetitions ($4b-$4d)
        ld      de, $4406
        call    reps            ; consolidate "path narrow" repetitions ($44-$4a)
        ld      de, $340d
        call    reps            ; consolidate "dark passage" repetitions ($34-$41)

        ld      l, a
        add     a, a
        add     a, l
        ld      l, a
        ld      h, $40
        add     hl, hl          ; calculate address $8000 + 6*A

        ld      a, (hl)         ; read RAM bank
        ld      bc, $7ffd
        push    bc
        out     (c), a          ; set RAM bank

        inc     hl
        ld      a, (hl)         ; read BORDER colour
        out     ($fe), a        ; set BORDER colour

        inc     hl
        ld      e, (hl)
        inc     hl
        ld      d, (hl)         ; read compressed attributes address
        push    de
        inc     hl
        ld      e, (hl)
        inc     hl
        ld      d, (hl)         ; read compressed pixel screen address
        ex      de, hl
        ld      de, $4000
        call    dzx7            ; decompress pixel screen
        pop     hl
        ld      de, $5800
        call    dzx7            ; decompress attributes

        pop     bc
        ld      a, $10
        out     (c), a          ; reset RAM bank 0

        pop     de
        pop     bc
        pop     hl
        pop     af              ; recover registers

        ret                     ; done!

reps:
        cp      d
        ret     c
        sub     e
        cp      d
        ret     nc
        ld      a, d
        ret


; -----------------------------------------------------------------------------
; Lookup table for compressed screens
; -----------------------------------------------------------------------------

        ORG     $8006

room01:                         ; tunnel like hall
        db      $10             ; RAM bank
        db      6               ; BORDER colour
        dw      att01           ; compressed attributes
        dw      pix01           ; compressed pixel screen

room02:                         ; road forest
        db      $10             ; RAM bank
        db      1               ; BORDER colour
        dw      att02           ; compressed attributes
        dw      pix02           ; compressed pixel screen

room03:                         ; forest of beech trees
        db      $10             ; RAM bank
        db      4               ; BORDER colour
        dw      att03           ; compressed attributes
        dw      pix03           ; compressed pixel screen

room04:                         ; lonelands
        db      $10             ; RAM bank
        db      6               ; BORDER colour
        dw      att04           ; compressed attributes
        dw      pix04           ; compressed pixel screen

room05:                         ; trolls clearing
        db      $10             ; RAM bank
        db      0               ; BORDER colour
        dw      att05night      ; compressed attributes
        dw      pix05           ; compressed pixel screen

room06:                         ; trolls path
        db      $10             ; RAM bank
        db      0               ; BORDER colour
        dw      att06           ; compressed attributes
        dw      pix06           ; compressed pixel screen

room07:                         ; trolls cave
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att07           ; compressed attributes
        dw      pix07           ; compressed pixel screen

room08:                         ; strong river
        db      $11             ; RAM bank
        db      3               ; BORDER colour
        dw      att08           ; compressed attributes
        dw      pix08           ; compressed pixel screen

room09:                         ; rivendell
        db      $11             ; RAM bank
        db      7               ; BORDER colour
        dw      att09           ; compressed attributes
        dw      pix09           ; compressed pixel screen

room0a:                         ; misty mountain
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0a           ; compressed attributes
        dw      pix0a           ; compressed pixel screen

room0b:                         ; narrow place
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0b           ; compressed attributes
        dw      pix0b           ; compressed pixel screen

room0c:                         ; narrow dangerous path
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0c           ; compressed attributes
        dw      pix0c           ; compressed pixel screen

room0d:                         ; goblins dungeon
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0d           ; compressed attributes
        dw      pix0d           ; compressed pixel screen

room0e:                         ; dry large cave
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0e           ; compressed attributes
        dw      pix0e           ; compressed pixel screen

room0f:                         ; stuffy dark passage
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0f           ; compressed attributes
        dw      pix0f           ; compressed pixel screen

room10:                         ; goblins big cavern
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att10           ; compressed attributes
        dw      pix10           ; compressed pixel screen

room11:                         ; dark deep lake
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att11           ; compressed attributes
        dw      pix11           ; compressed pixel screen

room12:                         ; winding dark passage
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att12           ; compressed attributes
        dw      pix12           ; compressed pixel screen

room13:                         ; goblins inside gate
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att13           ; compressed attributes
        dw      pix13           ; compressed pixel screen

room14:                         ; goblins outside gate
        db      $13             ; RAM bank
        db      5               ; BORDER colour
        dw      att14           ; compressed attributes
        dw      pix14           ; compressed pixel screen

room15:                         ; treeless opening
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att15           ; compressed attributes
        dw      pix15           ; compressed pixel screen

room16:                         ; Beorn's house
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att16           ; compressed attributes
        dw      pix16           ; compressed pixel screen

room17:                         ; river beneath mountains
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att17           ; compressed attributes
        dw      pix17           ; compressed pixel screen

room18:                         ; gate to mirkwood
        db      $13             ; RAM bank
        db      0               ; BORDER colour
        dw      att18           ; compressed attributes
        dw      pix18           ; compressed pixel screen

room19:                         ; gloomy bewitched place
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att19           ; compressed attributes
        dw      pix19           ; compressed pixel screen

room1a:                         ; threads spider place
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att1a           ; compressed attributes
        dw      pix1a           ; compressed pixel screen

room1b:                         ; smothering forest
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att1b           ; compressed attributes
        dw      pix1b           ; compressed pixel screen

room1c:                         ; elvish levelled clearing
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att1c           ; compressed attributes
        dw      pix1c           ; compressed pixel screen

room1d:                         ; deep bog
        db      $14             ; RAM bank
        db      4               ; BORDER colour
        dw      att1d           ; compressed attributes
        dw      pix1d           ; compressed pixel screen

room1e:                         ; great elvenking's halls
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att1e           ; compressed attributes
        dw      pix1e           ; compressed pixel screen

room1f:                         ; dark dungeon
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att1f           ; compressed attributes
        dw      pix1f           ; compressed pixel screen

room20:                         ; elvenking's cellar
        db      $14             ; RAM bank
        db      3               ; BORDER colour
        dw      att20           ; compressed attributes
        dw      pix20           ; compressed pixel screen

room21:                         ; forest river
        db      $14             ; RAM bank
        db      0               ; BORDER colour
        dw      att21           ; compressed attributes
        dw      pix21           ; compressed pixel screen

room22:                         ; long lake
        db      $14             ; RAM bank
        db      2               ; BORDER colour
        dw      att22           ; compressed attributes
        dw      pix22           ; compressed pixel screen

room23:                         ; lake town
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att23           ; compressed attributes
        dw      pix23           ; compressed pixel screen

room24:                         ; strong river (same as room08)
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att08           ; compressed attributes
        dw      pix08           ; compressed pixel screen

room25:                         ; bleak barren land
        db      $16             ; RAM bank
        db      2               ; BORDER colour
        dw      att25           ; compressed attributes
        dw      pix25           ; compressed pixel screen

room26:                         ; Dale town ruins
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att26           ; compressed attributes
        dw      pix26           ; compressed pixel screen

room27:                         ; front gate
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att27           ; compressed attributes
        dw      pix27           ; compressed pixel screen

room28:                         ; ravenhill
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att28           ; compressed attributes
        dw      pix28           ; compressed pixel screen

room29:                         ; dragon's halls
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att29           ; compressed attributes
        dw      pix29           ; compressed pixel screen

room2a:                         ; little steep bay
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att2a           ; compressed attributes
        dw      pix2a           ; compressed pixel screen

room2b:                         ; smooth straight passage
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att2b           ; compressed attributes
        dw      pix2b           ; compressed pixel screen

room2c:                         ; lonely mountain
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att2c           ; compressed attributes
        dw      pix2c           ; compressed pixel screen

room2d:                         ; waterfall
        db      $16             ; RAM bank
        db      0               ; BORDER colour
        dw      att2d           ; compressed attributes
        dw      pix2d           ; compressed pixel screen

room2e:                         ; forest path
        db      $17             ; RAM bank
        db      2               ; BORDER colour
        dw      att2e           ; compressed attributes
        dw      pix2e           ; compressed pixel screen

room2f:                         ; the end
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att2f           ; compressed attributes
        dw      pix2f           ; compressed pixel screen

room30:                         ; foot of mountain
        db      $17             ; RAM bank
        db      1               ; BORDER colour
        dw      att30           ; compressed attributes
        dw      pix30           ; compressed pixel screen

room31:                         ; river bank
        db      $17             ; RAM bank
        db      2               ; BORDER colour
        dw      att31           ; compressed attributes
        dw      pix31           ; compressed pixel screen

room32:                         ; forest of beech trees (same as room03)
        db      $10             ; RAM bank
        db      0               ; BORDER colour
        dw      att03           ; compressed attributes
        dw      pix03           ; compressed pixel screen

room33:                         ; barren empty place
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att33           ; compressed attributes
        dw      pix33           ; compressed pixel screen

room34_41:                      ; stuffy dark passage (same as room0f)
        db      $11             ; RAM bank
        db      0               ; BORDER colour
        dw      att0f           ; compressed attributes
        dw      pix0f           ; compressed pixel screen

room42:                         ; west bank
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att42           ; compressed attributes
        dw      pix42           ; compressed pixel screen

room43:                         ; east bank
        db      $17             ; RAM bank
        db      4               ; BORDER colour
        dw      att43           ; compressed attributes
        dw      pix43           ; compressed pixel screen

room44_4a:                      ; narrow path
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att44           ; compressed attributes
        dw      pix44           ; compressed pixel screen

room4b_4d:                      ; steep path
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att4b           ; compressed attributes
        dw      pix4b           ; compressed pixel screen

room4e_4f:                      ; misty deep valley 
        db      $17             ; RAM bank
        db      0               ; BORDER colour
        dw      att4e           ; compressed attributes
        dw      pix4e           ; compressed pixel screen

dzx7:
include "dzx7_smartRCS.asm"     ; decompressor


; -----------------------------------------------------------------------------
; Area $c11b-$c730 used for storing objects table, although it can be
; temporarily reused for something else before game starts
; -----------------------------------------------------------------------------

        ORG     $c11b

credits:
incbin  "credits.scr.rcs.zx7"   ; credits screen


; -----------------------------------------------------------------------------
; Area $cc00-$f3ff originally used for storing graphics, now replaced with
; compressed screens.
; -----------------------------------------------------------------------------

        ORG $cc00

pix01:
incbin  "pix01.scr.rcs.zx7"
att01:
incbin  "att01.scr.zx7"

pix02:
incbin  "pix02.scr.rcs.zx7"
att02:
incbin  "att02.scr.zx7"

pix03:
incbin  "pix03.scr.rcs.zx7"
att03:
incbin  "att03.scr.zx7"

pix04:
incbin  "pix04.scr.rcs.zx7"
att04:
incbin  "att04.scr.zx7"

pix05:
incbin  "pix05night.scr.rcs.zx7"
att05night:
incbin  "att05night.scr.zx7"
att05day:
incbin  "att05day.scr.zx7"

pix06:
incbin  "pix06.scr.rcs.zx7"
att06:
incbin  "att06.scr.zx7"

objs:
incbin  "objs_49435.bin.zx7"

locs:
incbin  "locs_47754.bin.zx7"

; -----------------------------------------------------------------------------
